@@ -28,6 +28,7 @@ import gobject
 import gst
 import objectfactory
 
+from gettext import gettext as _
 import os.path
 
 class Discoverer(gobject.GObject):
@@ -35,14 +36,17 @@ class Discoverer(gobject.GObject):
     Queues requests to discover information about given files.
     The discovery is done in a very fragmented way, so that it appears to be
     running in a separate thread.
-    
+
     The "new_sourcefilefactory" signal is emitted when a file is established
     to be a media_file and the FileSourceFactory() is included in the signal.
-    
+
     The "not_media_file" signal is emitted if a file is not a media_file.
 
     The "finished-analyzing" signal is emitted a file is finished being analyzed
 
+    The "starting" signal isemitted when the discoverer starts analyzing some
+    files.
+
     The "ready" signal is emitted when the discoverer has no more files to
     analyze.
     """
@@ -53,13 +57,16 @@ class Discoverer(gobject.GObject):
                                    (gobject.TYPE_PYOBJECT, )),
         "not_media_file" : (gobject.SIGNAL_RUN_LAST,
                             gobject.TYPE_NONE,
-                            (gobject.TYPE_STRING, gobject.TYPE_STRING)),
+                            (gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)),
         "finished_analyzing" : ( gobject.SIGNAL_RUN_LAST,
                                  gobject.TYPE_NONE,
                                  (gobject.TYPE_PYOBJECT, )),
         "ready" : ( gobject.SIGNAL_RUN_LAST,
                     gobject.TYPE_NONE,
-                    ( ))
+                    ( )),
+        "starting" : ( gobject.SIGNAL_RUN_LAST,
+                       gobject.TYPE_NONE,
+                       ( ))
         }
 
     def __init__(self, project):
@@ -79,6 +86,8 @@ class Discoverer(gobject.GObject):
         self.nomorepads = False
         self.timeoutid = 0
         self.signalsid = []
+        self.error = None # reason for error
+        self.extrainfo = None # extra information about the error
         self.fakesink = None
 
     def addFile(self, filename):
@@ -102,12 +111,13 @@ class Discoverer(gobject.GObject):
         if self.working:
             gst.warning("called when still working!")
             return False
-        
+
         if not self.queue:
             gst.warning("Nothing to analyze!!")
             return False
-        
+
         self.working = True
+        self.emit("starting")
         gobject.idle_add(self._analyze)
         return False
 
@@ -139,9 +149,16 @@ class Discoverer(gobject.GObject):
         gst.info("after setting to NULL : %s" % res)
         if self.fakesink:
             self.fakesink.set_state(gst.STATE_NULL)
-        if self.currentfactory:
+        if self.error:
+            self.emit('not_media_file', self.current, self.error, self.extrainfo)
+        elif self.currentfactory:
             self.currentfactory.addMediaTags(self.currentTags)
-            self.emit('finished-analyzing', self.currentfactory)
+            if not self.currentfactory.length:
+                self.emit('not_media_file', self.current,
+                          _("Could not establish the duration of the file."),
+                          _("This clip seems to be in a format which cannot be accessed in a random fashion."))
+            else:
+                self.emit('finished-analyzing', self.currentfactory)
         self.currentTags = []
         self.analyzing = False
         self.current = None
@@ -150,7 +167,9 @@ class Discoverer(gobject.GObject):
         self.fakesink = None
         self.prerolled = False
         self.nomorepads = False
-        
+        self.error = None
+        self.extrainfo = None
+
         # restart an analysis if there's more...
         if self.queue:
             gobject.idle_add(self._analyze)
@@ -164,7 +183,7 @@ class Discoverer(gobject.GObject):
         gst.debug("timeout")
         gobject.idle_add(self._finishAnalysis)
         return False
-    
+
     def _analyze(self):
         """
         Sets up a pipeline to analyze the given uri
@@ -174,13 +193,15 @@ class Discoverer(gobject.GObject):
         self.current = self.queue.pop(0)
         gst.info("Analyzing %s" % self.current)
         self.currentfactory = None
-        
+
         # setup graph and start analyzing
         self.pipeline = gst.Pipeline("Discoverer-%s" % self.current)
         source = gst.element_make_from_uri(gst.URI_SRC, self.current, "src-%s" % self.current)
         if not source:
             gst.warning("This is not a media file : %s" % self.current)
-            self.emit("not_media_file", self.current, "Couldn't construct pipeline.")
+            if not self.error:
+                self.error = _("Couldn't construct pipeline.")
+                self.extrainfo = _("GStreamer does not have an element to handle files coming from this type of file system.")
             gobject.idle_add(self._finishAnalysis)
             return False
         if os.getenv("USE_DECODEBIN2"):
@@ -197,14 +218,15 @@ class Discoverer(gobject.GObject):
         # adding fakesink to make pipeline not terminate state before receiving no-more-pads
         self.fakesink = gst.element_factory_make("fakesink")
         self.pipeline.add(self.fakesink)
-        
+
         self.bus = self.pipeline.get_bus()
         self.signalsid.append((self.bus, self.bus.connect("message", self._busMessageCb)))
         self.bus.add_signal_watch()
 
         gst.info("setting pipeline to PAUSED")
         if self.pipeline.set_state(gst.STATE_PAUSED) == gst.STATE_CHANGE_FAILURE:
-            self.emit("not_media_file", self.current, "Pipeline didn't want to go to PAUSED")
+            if not self.error:
+                self.error = _("Pipeline didn't want to go to PAUSED.")
             gst.info("pipeline didn't want to go to PAUSED")
             gobject.idle_add(self._finishAnalysis)
             return False
@@ -214,7 +236,7 @@ class Discoverer(gobject.GObject):
 
         # return False so we don't get called again
         return False
-        
+
     def _busMessageCb(self, unused_bus, message):
         if self.thisdone:
             return
@@ -231,7 +253,8 @@ class Discoverer(gobject.GObject):
                     if self.currentfactory and self.currentfactory.is_video:
                         gst.log("pipeline has gone to PAUSED, now pushing to PLAYING")
                         if self.pipeline.set_state(gst.STATE_PLAYING) == gst.STATE_CHANGE_FAILURE:
-                            self.emit("not_media_file", self.current, "Pipeline didn't want to go to PLAYING")
+                            if not self.error:
+                                self.error = _("Pipeline didn't want to go to PLAYING.")
                             gst.info("Pipeline didn't want to go to playing")
                             gobject.idle_add(self._finishAnalysis)
                     elif self.nomorepads:
@@ -255,6 +278,9 @@ class Discoverer(gobject.GObject):
             gst.debug("Element message %s" % message.structure.to_string())
             if message.structure.get_name() == "redirect":
                 gst.warning("We don't implement redirections currently, ignoring file")
+                if not self.error:
+                    self.error = _("File contains a redirection to another clip.")
+                    self.extrainfo = _("PiTiVi does not currently does not handle redirection files.")
                 gobject.idle_add(self._finishAnalysis)
         elif message.type == gst.MESSAGE_TAG:
             gst.debug("Got tags %s" % message.structure.to_string())
@@ -262,11 +288,13 @@ class Discoverer(gobject.GObject):
         else:
             gst.log("%s:%s" % ( message.type, message.src))
 
-    def _handleError(self, gerror, unused_detail, unused_source):
+    def _handleError(self, gerror, detail, unused_source):
         gst.warning("got an ERROR")
         
-        self.emit("not_media_file", self.current, "An error occured while analyzing this file")
-        self.emit("not_media_file", self.current, gerror.message)
+        if not self.error:
+            self.error = _("An internal error occured while analyzing this file : %s") % gerror.message
+            self.extrainfo = detail
+
         self.thisdone = True
         self.currentfactory = None
         gobject.idle_add(self._finishAnalysis)
@@ -321,11 +349,11 @@ class Discoverer(gobject.GObject):
         pngenc = gst.element_factory_make("pngenc")
         pngsink = gst.element_factory_make("filesink")
         pngsink.set_property("location", "/tmp/" + self.currentfactory.name.encode('base64').replace('\n','') + ".png")
-        
+
         self.pipeline.add(q, csp, pngenc, pngsink)
         gst.element_link_many(q, csp, pngenc, pngsink)
         pad.link(q.get_pad("sink"))
-        
+
         if not self.currentfactory.video_info:
             self.signalsid.append((pad, pad.connect("notify::caps", self._vcapsNotifyCb)))
 
@@ -335,7 +363,7 @@ class Discoverer(gobject.GObject):
         if self.currentfactory.is_audio:
             gst.debug("already have audio, calling no_more_pads")
             self._noMorePadsCb(None)
-        
+
     def _newAudioPadCb(self, unused_element, pad):
         """ a new audio pad was found """
         gst.debug("pad %s" % pad)
@@ -346,7 +374,7 @@ class Discoverer(gobject.GObject):
         if self.currentfactory.is_video:
             gst.debug("already have video, calling no_more_pads")
             self._noMorePadsCb(None)
-            
+
         if pad.get_caps().is_fixed():
             gst.debug("fixed caps, setting info on factory")
             self.currentfactory.setAudioInfo(pad.get_caps())
@@ -367,7 +395,9 @@ class Discoverer(gobject.GObject):
         gst.info(caps.to_string())
         if not self.currentfactory or (not self.currentfactory.is_audio and not self.currentfactory.is_video):
             gst.warning("got unknown pad without anything else")
-            self.emit("not_media_file", self.current, "Got unknown stream type : %s" % caps.to_string())
+            if not self.error:
+                self.error = _("Got unknown stream type : %s") % caps.to_string()
+                self.extrainfo = _("You are missing an element to handle this media type.")
             gobject.idle_add(self._finishAnalysis)
 
     def _newDecodedPadCb(self, element, pad, is_last):
@@ -390,7 +420,9 @@ class Discoverer(gobject.GObject):
             if is_last:
                 if not self.currentfactory or not self.currentfactory.is_audio or not self.currentfactory.is_video:
                     gst.warning("couldn't find a usable pad")
-                    self.emit("not_media_file", self.current, "Got unknown stream type : %s" % capsstr)
+                    if not self.error:
+                        self.error = "Got unknown stream type : %s" % capsstr
+                        self.extrainfo = _("You are missing an element to handle this media type.")
                     gobject.idle_add(self._finishAnalysis)
 
     def _noMorePadsCb(self, unused_element):
